#!/usr/bin/env python
import os
import sys
import xml.etree.ElementTree as ET

#add lib path
playbook_file = os.path.realpath(sys.argv[0])
playbook_path=os.path.split(playbook_file)[0]   

lib_path=os.path.join(playbook_path,'library')
if os.path.exists(lib_path) is True:
    sys.path.insert(0,lib_path)
    lib_path=os.path.join(lib_path,'securecat')
    if os.path.exists(lib_path) is True:
        sys.path.insert(0,lib_path)
        modulepath = os.environ.get('CSC_MODULE_PATH')
        if modulepath is None:
            os.environ['CSC_MODULE_PATH'] = lib_path

from utils import playbook

class MyPlaybook(playbook.Playbook):
    
    def set_extra_vars_pre(self):
        self._runcontext.set_extra_var('TOMCAT_HOME','/usr/local/tomcat7')
        self._runcontext.set_extra_var('TOMCAT_BASE','/usr/local/tomcat7')
    
    def set_extra_vars_post(self):
        errorMsg = ''
        # hostguard write "CWD" in param.json, rewrite TOMCAT_HOME and TOMCAT_BASE;
        cwd = os.path.abspath(os.path.dirname(os.path.dirname(self._runcontext.get_extra_var('CONF_PATH'))))
        self._runcontext.set_extra_var('TOMCAT_HOME',cwd)
        self._runcontext.set_extra_var('TOMCAT_BASE',cwd)
        tomcat_base = self._runcontext.get_extra_var('TOMCAT_BASE')
        
        serverPath = os.path.join(tomcat_base,'conf','server.xml')
        if os.path.exists(serverPath) is False:
            errorMsg = 'File %s is not exists.' % serverPath
            raise Exception(errorMsg)        
        self._runcontext.set_extra_var('TOMCAT_GLOBAL_SERVER_FILE',serverPath)
        
        tomcat_user = os.path.join(tomcat_base,'conf','tomcat-users.xml')
        if os.path.exists(tomcat_user) is True:
            self._runcontext.set_extra_var('TOMCAT_GLOBAL_USER_FILE',tomcat_user)
        else:
            self._runcontext.set_extra_var('TOMCAT_GLOBAL_USER_FILE','')
        
        default_context = os.path.join(tomcat_base,'conf','context.xml')
        if os.path.exists(default_context) is True:
            self._runcontext.set_extra_var('TOMCAT_GLOBAL_CONTEXT_FILE',default_context)
        else:
            self._runcontext.set_extra_var('TOMCAT_GLOBAL_CONTEXT_FILE','')
            
        default_web = os.path.join(tomcat_base,'conf','web.xml')
        if os.path.exists(default_web) is True:
            self._runcontext.set_extra_var('TOMCAT_GLOBAL_WEB_FILE',default_web)
        else:
            self._runcontext.set_extra_var('TOMCAT_GLOBAL_WEB_FILE','')
        
        default_logging_properties = os.path.join(tomcat_base,'conf','logging.properties')
        if os.path.exists(default_logging_properties) is True:
            self._runcontext.set_extra_var('TOMCAT_GLOBAL_LOGGING_PROPERTIES_FILES',default_logging_properties)
        else:
            self._runcontext.set_extra_var('TOMCAT_GLOBAL_LOGGING_PROPERTIES_FILES','')

        tomcat_catalina_policy = os.path.join(tomcat_base,'conf','catalina.policy')
        if os.path.exists(tomcat_catalina_policy) is True:
            self._runcontext.set_extra_var('TOMCAT_GLOBAL_CATALINA_POLICY_FILE',tomcat_catalina_policy)
        else:
            self._runcontext.set_extra_var('TOMCAT_GLOBAL_CATALINA_POLICY_FILE','')

        tomcat_catalina_properties = os.path.join(tomcat_base,'conf','catalina.properties')
        if os.path.exists(tomcat_catalina_properties) is True:
            self._runcontext.set_extra_var('TOMCAT_GLOBAL_CATALINA_PROPERTIES_FILE',tomcat_catalina_properties)
        else:
            self._runcontext.set_extra_var('TOMCAT_GLOBAL_CATALINA_PROPERTIES_FILE','')
        
        xmlTree = ET.parse(serverPath)
        root = xmlTree.getroot()
        v_webapps = []
        
        #loop Server
        for n1 in root:
            if n1.tag != 'Service':
                continue
                
            service_name = n1.get('name')
            for n2 in n1:
                if n2.tag != 'Engine':
                    continue
                
                engine_name = n2.get('name')
                
                for n3 in n2:
                    if n3.tag != 'Host':
                        continue
                    
                    host_name = n3.get('name')
                    host_appbase = n3.get('appBase')
                    host_autodeploy = n3.get('autoDeploy')
                    
                    if host_appbase == None or host_appbase.strip()=='':
                        host_appbase = "webapps"
                    
                    flag_has_context = False
                    
                    if host_autodeploy == None or host_autodeploy.strip() == 'true' or host_autodeploy.strip() == '':
                        flag_has_context = True
                        find_webapps_dir = os.path.join(tomcat_base, host_appbase)
                        find_webapps_list = os.listdir(find_webapps_dir)
                        for webapp_name in find_webapps_list:
                        
                            v_webapp = {}
                            v_webapp['service_name'] = service_name
                            v_webapp['engine_name'] = engine_name
                            v_webapp['host_name'] = host_name
                            v_webapp['host_appbase'] = host_appbase
                            v_webapp['context_docBase'] = os.path.join(find_webapps_dir, webapp_name)
                            v_webapp['context_path'] = os.path.join(find_webapps_dir, webapp_name)
                            
                            v_tomcat_web_path = []
                            v_tomcat_context_path = []
                            v_tomcat_properties_path = []
                            
                            contextxmlpath = os.path.join(tomcat_base,host_appbase,webapp_name,'context.xml')
                            if os.path.exists(contextxmlpath) is True:
                                v_tomcat_context_path.append(contextxmlpath)
                            contextxmlpath = os.path.join(tomcat_base,host_appbase,webapp_name,'META-INF','context.xml')
                            if os.path.exists(contextxmlpath) is True:
                                v_tomcat_context_path.append(contextxmlpath)
                            v_tomcat_context_path.append(default_context)
                                
                            webxmlpath = os.path.join(tomcat_base,host_appbase,webapp_name,'WEB-INF','web.xml')
                            if os.path.exists(webxmlpath) is True:
                                v_tomcat_web_path.append(webxmlpath)
                            v_tomcat_web_path.append(default_web)
                            
                            loggingcfgpath = os.path.join(tomcat_base,host_appbase,webapp_name,'WEB-INF','classes','logging.properties')
                            if os.path.exists(loggingcfgpath) is True:
                                v_tomcat_properties_path.append(loggingcfgpath)
                            v_tomcat_properties_path.append(default_logging_properties)
                            
                            while '' in v_tomcat_context_path:
                                v_tomcat_context_path.remove('')
                            
                            while '' in v_tomcat_web_path:
                                v_tomcat_web_path.remove('')
                                
                            while '' in v_tomcat_properties_path:
                                v_tomcat_properties_path.remove('')
                                
                            v_webapp['web.xml'] = v_tomcat_web_path
                            v_webapp['context.xml'] = v_tomcat_context_path
                            v_webapp['logging.properties'] = v_tomcat_properties_path
                            v_webapps.append(v_webapp)
                            
                    else:
                        for n4 in n3:
                            if n4.tag != 'Context':
                                continue
                            
                            flag_has_context = True
                            
                            v_tomcat_web_path = []
                            v_tomcat_context_path = []
                            v_tomcat_properties_path = []
                            
                            context_docBase = n4.get('docBase')
                            context_path = n4.get('path')
                            
                            if context_docBase == None or context_docBase.strip() == '':
                                context_docBase = "ROOT"
                            
                            v_webapp = {}
                            v_webapp['service_name'] = service_name
                            v_webapp['engine_name'] = engine_name
                            v_webapp['host_name'] = host_name
                            v_webapp['host_appbase'] = host_appbase
                            
                            webapp_dir = ''
                            if context_docBase.startswith('/'):
                                webapp_dir = context_docBase
                            else:
                                webapp_dir = os.path.join(tomcat_base,host_appbase,context_docBase)
                            v_webapp['context_docBase'] = webapp_dir
                            v_webapp['context_path'] = context_path
                            
                            contextxmlpath = os.path.join(webapp_dir,'context.xml')
                            if os.path.exists(contextxmlpath) is True:
                                v_tomcat_context_path.append(contextxmlpath)
                            contextxmlpath = os.path.join(webapp_dir,'META-INF','context.xml')
                            if os.path.exists(contextxmlpath) is True:
                                v_tomcat_context_path.append(contextxmlpath)
                            v_tomcat_context_path.append(default_context)
                                
                            webxmlpath = os.path.join(webapp_dir,'WEB-INF','web.xml')
                            if os.path.exists(webxmlpath) is True:
                                v_tomcat_web_path.append(webxmlpath)
                            v_tomcat_web_path.append(default_web)
                            
                            loggingcfgpath = os.path.join(webapp_dir,'WEB-INF','classes','logging.properties')
                            if os.path.exists(loggingcfgpath) is True:
                                v_tomcat_properties_path.append(loggingcfgpath)
                            v_tomcat_properties_path.append(default_logging_properties)
                            
                            while '' in v_tomcat_context_path:
                                v_tomcat_context_path.remove('')
                            
                            while '' in v_tomcat_web_path:
                                v_tomcat_web_path.remove('')
                                
                            while '' in v_tomcat_properties_path:
                                v_tomcat_properties_path.remove('')
                                
                            v_webapp['web.xml'] = v_tomcat_web_path
                            v_webapp['context.xml'] = v_tomcat_context_path
                            v_webapp['logging.properties'] = v_tomcat_properties_path
                            v_webapps.append(v_webapp)
                        
                    if flag_has_context is False:
                        v_webapp = {}
                        v_webapp['service_name'] = service_name
                        v_webapp['engine_name'] = engine_name
                        v_webapp['host_name'] = host_name
                        v_webapp['host_appbase'] = host_appbase
                        v_webapp['context_docBase'] = 'default'
                        v_webapp['context_path'] = ''
                        
                        v_tomcat_context_path = [ default_context ]
                        v_tomcat_web_path = [ default_web ]
                        v_tomcat_properties_path = [ default_logging_properties ]
                        
                        while '' in v_tomcat_context_path:
                            v_tomcat_context_path.remove('')
                        
                        while '' in v_tomcat_context_path:
                            v_tomcat_context_path.remove('')
                            
                        while '' in v_tomcat_context_path:
                            v_tomcat_context_path.remove('')
                        
                        v_webapp['web.xml'] = v_tomcat_web_path
                        v_webapp['context.xml'] = v_tomcat_context_path
                        v_webapp['logging.properties'] = v_tomcat_properties_path
                        v_webapps.append(v_webapp)
        
        self._runcontext.set_extra_var('TOMCAT_WEBAPPS',v_webapps)
        
    def get_tasks(self, dir):
        ret = []        

        ret.append([dir, 'r_2_1_2.py'])
        ret.append([dir, 'r_2_1_3_1.py'])
        ret.append([dir, 'r_2_1_4.py'])
        ret.append([dir, 'r_2_1_5.py'])
        ret.append([dir, 's_2_1_2.py'])
        ret.append([dir, 's_2_1_3.py'])

        ret.append([dir, 'r_2_2_1.py'])
        ret.append([dir, 'r_2_2_2.py'])
        ret.append([dir, 'r_2_2_3.py'])
        ret.append([dir, 'r_2_2_4.py'])
        ret.append([dir, 'r_2_2_6.py'])
        ret.append([dir, 'r_2_2_7.py'])
        ret.append([dir, 'r_2_2_8.py'])
        ret.append([dir, 'r_2_2_10.py'])
        ret.append([dir, 'r_2_2_11.py'])
        ret.append([dir, 'r_2_2_12.py'])
        ret.append([dir, 'r_2_2_13.py'])
        ret.append([dir, 'r_2_2_15.py'])
        ret.append([dir, 'r_2_2_16.py'])
        ret.append([dir, 'r_2_2_17.py'])
        ret.append([dir, 'r_2_2_18.py'])
        ret.append([dir, 'r_2_2_19.py'])
        ret.append([dir, 'r_2_2_20.py'])
        ret.append([dir, 'r_2_2_22.py'])
        ret.append([dir, 'r_2_2_24.py'])
        ret.append([dir, 's_2_2_1.py'])

        ret.append([dir, 'r_2_3_1.py'])
        ret.append([dir, 'r_2_3_2.py'])
        ret.append([dir, 'r_2_3_3.py'])
        ret.append([dir, 'r_2_3_4.py'])
        ret.append([dir, 'r_2_3_5.py'])
        ret.append([dir, 'r_2_3_6.py'])
        ret.append([dir, 's_2_3_1.py'])

        ret.append([dir, 'r_2_4_1.py'])

        return ret
        
if __name__ == '__main__':
    
    myplaybook = MyPlaybook()
    myplaybook.set_role('hw')
    myplaybook.run()
    

