# -*- coding: UTF-8 -*-
import os
import json
import xml.etree.ElementTree as ET


def get_tags():
    return ['2', '2.1', '2.1.3', '2.1.3.2']


def run_task(runcontext):
    ret_code = -1
    ret_msg = ''
    curtemplate = runcontext.get_template()
    tomcatbase = runcontext.get_extra_var('TOMCAT_BASE')
    tomcatbase = runcontext.replace_inject_characters(tomcatbase)
    runcontext.log_debug("tomcatbase: %s" % tomcatbase)

    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''

    # check 1
    pConf = os.path.join(tomcatbase.rstrip(os.sep), 'conf')
    pConfTomcatUsers = os.path.join(pConf.rstrip(os.sep), 'tomcat-users.xml')
    runcontext.log_debug("pConf: %s" % pConf)
    if False == os.path.exists(pConfTomcatUsers):
        tmp_vars['checkvalue1'] = "Not exist file tomcat-users.xml"
        tmp_vars["checkresult1"] = "pass"
    else:
        ansible_params = runcontext.get_mod_params_instance()
        runcontext.set_mod_params_instance(ansible_params,
                                           '_ansible_module_name', 'xml_query')
        runcontext.set_mod_params_instance(ansible_params, 'action',
                                           runcontext.get_parameter('action'))
        runcontext.set_mod_params_instance(ansible_params, 'backupfile',
                                           os.path.join(
                                               runcontext.get_parameter(
                                                   'backup_path'),
                                               'r_2_1_3_1.json'))
        runcontext.set_mod_params_instance(ansible_params, 'path',
                                           pConfTomcatUsers)

        runcontext.set_mod_params_instance(ansible_params, 'xpath',
                                           './/{ns}user')
        runcontext.set_mod_params_instance(ansible_params, 'content',
                                           'attribute')
        runcontext.set_mod_params_instance(ansible_params, 'attribute',
                                           'password')
        runcontext.set_mod_params_instance(ansible_params, 'operation',
                                           'match')
        runcontext.set_mod_params_instance(ansible_params, 'value', '.+')

        module = runcontext.get_module_file('xml_query.py')
        rc1, stdout1, stderr1 = runcontext.invoke_module(module, ansible_params)
        runcontext.sleep()

        runcontext.set_mod_params_instance(ansible_params, 'attribute',
                                           'username')
        runcontext.set_mod_params_instance(ansible_params, 'value', '.+')
        module = runcontext.get_module_file('xml_query.py')
        rc2, stdout2, stderr2 = runcontext.invoke_module(module, ansible_params)
        runcontext.sleep()

        resultjson1 = json.loads(stdout1)
        resultjson2 = json.loads(stdout2)

        runcontext.log_debug_json(resultjson1)
        runcontext.log_debug_json(resultjson2)

        # run failed
        if 'failed' in resultjson1:
            ret_code = -2
            return ret_code, resultjson1['msg']
        if 'failed' in resultjson2:
            ret_code = -2
            return ret_code, resultjson2['msg']

        tmp_vars = {}

        passwd_num = len(resultjson1['items']['values'])
        if 1 == len(resultjson1['items']['values']) and '' == resultjson1['items']['values'][0]:
            tmp_vars["checkresult1"] = "pass"
            tmp_vars['checkvalue1'] = "No role and password are configured"
        else:

            msg_fail = []
            # 判断密码复杂度是否符合要求
            score = 0
            for num, password in enumerate(resultjson1['items']['values']):
                if len(password) < 8:
                    msg_fail.append(resultjson2['items']['values'][num])
                    continue
                lst = [0, 0, 0, 0]
                for i in password:
                    if sum(lst) >= 2:
                        score += 1
                        break
                    if i.isupper():
                        lst[0] = 1
                    elif i.islower():
                        lst[1] = 1
                    elif i.isdigit():
                        lst[2] = 1
                    else:
                        lst[3] = 1

                if sum(lst) < 2:
                    msg_fail.append(resultjson2['items']['values'][num])

            if passwd_num == score:
                tmp_vars["checkresult1"] = "pass"
                tmp_vars[
                    'checkvalue1'] = "All passwords meet the complexity requirements"
            else:
                tmp_vars["checkresult1"] = "failed"
                fail_msg = "Users who do not meet the password conditions: "
                tmp_vars['checkvalue1'] = fail_msg + ", ".join(msg_fail)

    if tmp_vars["checkresult1"] == "pass":
        tmp_vars["policyresult"] = "pass"
        ret_code = 0

    rc, out = curtemplate.render(runcontext.get_template_file('r_2_1_3_2.j2'),
                                 **tmp_vars)

    if rc == False:
        ret_code = -2
        ret_msg = ('Failed to render template. %s' % out)
    else:
        ret_msg = out

    return ret_code, ret_msg
