# -*- coding: UTF-8 -*-
import re
try:
    import xml.etree.cElementTree as ET
except ImportError:
    import xml.etree.ElementTree as ET


def get_tags():
    return ['2', '2.2', '2.2.4']


def get_xml_namespace(node):
    pattern = re.compile(r'({\S+})(\S+)')
    m = pattern.match(node)
    if m:
        return m.groups()[0]
    else:
        return ''


def get_node_tag(node):
    pattern = re.compile(r'({\S+})*(\S+)')
    m = pattern.match(node)
    if m:
        return m.groups()[1]
    else:
        return ''


def run_task(runcontext):
    gwebxml = runcontext.get_extra_var('TOMCAT_GLOBAL_WEB_FILE')

    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["systemvalue1"] = ''
    ret_code = -1
    ret_msg = ''
    ret_err_msg = []
    ret_ok_msg = []

    xmlTree = ET.parse(gwebxml)
    root = xmlTree.getroot()
    namespace = get_xml_namespace(root.tag)
    s = './{0}error-page'.format(namespace)
    err_page_nodes = root.findall(s)
    ret_ok_msg.append('<web-app>')
    ret_err_msg.append('<web-app>')
    if err_page_nodes:
        all_result = []
        for node in err_page_nodes:
            node_msg = []
            node_tag = get_node_tag(node.tag)
            node_msg.append('    <{0}>'.format(node_tag))
            sons_result = []
            son_msg = ''
            for son in list(node):
                son_tag = get_node_tag(son.tag)
                son_msg = '        <{0}>{1}</{0}>'.format(son_tag, son.text)
                if son.text:
                    sons_result.append(True)
                else:
                    sons_result.append(False)

            node_msg.append(son_msg)
            node_msg.append('    </{0}>'.format(node_tag))
            if False not in sons_result:
                all_result.append(True)
                ret_ok_msg.extend(node_msg)
            else:
                all_result.append(False)
                ret_err_msg.extend(node_msg)

        if False not in all_result:
            ret_code = 0
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["policyresult"] = "pass"
            ret_ok_msg.append('</web-app>')
            tmp_vars["systemvalue1"] = '\n'.join(ret_ok_msg)
        else:
            ret_err_msg.append('</web-app>')
            tmp_vars["systemvalue1"] = '\n'.join(ret_err_msg)
    else:
        tmp_vars["systemvalue1"] = 'error-page is not configured'

    curtemplate = runcontext.get_template()
    rc1, out = curtemplate.render(runcontext.get_template_file('r_2_2_4.j2'), **tmp_vars)

    if rc1 == False:
        ret_code = -2
        ret_msg = ('Failed to render template. %s' % out)
    else:
        ret_msg = out

    return ret_code, ret_msg
