# -*- coding: UTF-8 -*-
import re
try:
    import xml.etree.cElementTree as ET
except ImportError:
    import xml.etree.ElementTree as ET


def get_tags():
    return ['2', '2.2', '2.2.22']


def get_xml_namespace(node):
    pattern = re.compile(r'({\S+})(\S+)')
    m = pattern.match(node)
    if m:
        return m.groups()[0]
    else:
        return ''


def get_node_tag(node):
    pattern = re.compile(r'({\S+})*(\S+)')
    m = pattern.match(node)
    if m:
        return m.groups()[1]
    else:
        return ''


def run_task(runcontext):
    g_server_xml = runcontext.get_extra_var('TOMCAT_GLOBAL_SERVER_FILE')

    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["systemvalue1"] = ''
    ret_code = -1
    ret_msg = ''
    ret_err_msg = []
    ret_ok_msg = []

    xmlTree = ET.parse(g_server_xml)
    root = xmlTree.getroot()
    namespace = get_xml_namespace(root.tag)
    s = './/{0}Connector'.format(namespace)
    connector_nodes = root.findall(s)
    ret_ok_msg.append('<Server>\n...')
    ret_err_msg.append('<Server>\n...')
    if connector_nodes:
        all_result = []
        for node in connector_nodes:
            node_msg = []
            node_tag = get_node_tag(node.tag)
            attribs_result = [False, False, False, False]
            attrib_list = []
            http_flag = False
            node_msg.append('    <{0} '.format(node_tag))
            for attrib in node.attrib.keys():
                attrib_value = node.attrib.get(attrib)
                attrib_value = '' if attrib_value is None else attrib_value
                attrib_list.append('{0}="{1}"'.format(attrib, attrib_value))
                node_msg.append('{0}="{1}"'.format(attrib, attrib_value))
                if attrib == 'SSLEnabled':
                    if attrib_value == 'true':
                        attribs_result[0] = True
                    else:
                        if attribs_result[0]:
                            pass
                if attrib == 'scheme':
                    if attrib_value == 'https':
                        attribs_result[1] = True
                    else:
                        if attribs_result[1]:
                            pass
                if attrib == 'secure':
                    if attrib_value == 'true':
                        attribs_result[2] = True
                    else:
                        if attribs_result[2]:
                            pass
                if attrib == 'sslEnabledProtocols' or attrib == 'sslProtocol':
                    attrib_list = attrib_value.split(',')
                    if 'TLSv1.1' in attrib_list or 'TLSv1.2' in attrib_list:
                        attribs_result[3] = True
                    else:
                        if attribs_result[3]:
                            pass
                            
                if attrib == 'protocol' and "http" in attrib_value.lower():
                    http_flag = True

            node_msg.append('>\n    </{0}>'.format(node_tag))
            if False not in attribs_result and http_flag:
                all_result.append(True)
                ret_ok_msg.append(' '.join(node_msg))
            elif http_flag == False:
                pass
            else:
                all_result.append(False)
                ret_err_msg.append(' '.join(node_msg))

        if True in all_result:
            ret_code = 0
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["policyresult"] = "pass"
            ret_ok_msg.append('...\n</Server>')
            tmp_vars["systemvalue1"] = '\n'.join(ret_ok_msg)
        else:
            ret_err_msg.append('...\n</Server>')
            tmp_vars["systemvalue1"] = '\n'.join(ret_err_msg)
    else:
        ret_code = 0
        tmp_vars["systemvalue1"] = 'Connector is not configured'
        tmp_vars["checkresult1"] = "pass"
        tmp_vars["policyresult"] = "pass"

    curtemplate = runcontext.get_template()
    rc1, out = curtemplate.render(runcontext.get_template_file('r_2_2_22.j2'), **tmp_vars)

    if rc1 == False:
        ret_code = -2
        ret_msg = ('Failed to render template. %s' % out)
    else:
        ret_msg = out

    return ret_code, ret_msg
