# -*- coding: UTF-8 -*-
import os
import json

try:
    import xml.etree.cElementTree as ET
except ImportError:
    import xml.etree.ElementTree as ET


def get_tags():
    return ['2', '2.2', '2.2.3']


def run_task(runcontext):
    ansible_params = runcontext.get_mod_params_instance()
    rc, out = run_audit(runcontext, ansible_params)
    return rc, out


def run_audit(runcontext, ansible_params):
    errMsg = ''
    runcontext.set_mod_params_instance(ansible_params, '_ansible_module_name', 'xml_query')
    runcontext.set_mod_params_instance(ansible_params, 'action', runcontext.get_parameter('action'))
    runcontext.set_mod_params_instance(ansible_params, 'backupfile', os.path.join(runcontext.get_parameter('backup_path'), 'r_2_2_3.json'))
    gserverxml = runcontext.get_extra_var('TOMCAT_GLOBAL_SERVER_FILE')
    gcontextxml = runcontext.get_extra_var('TOMCAT_GLOBAL_CONTEXT_FILE')

    runcontext.set_mod_params_instance(ansible_params, 'path', gserverxml)
    runcontext.set_mod_params_instance(ansible_params, 'xpath', './/{ns}Context')
    runcontext.set_mod_params_instance(ansible_params, 'content', 'attribute')
    runcontext.set_mod_params_instance(ansible_params, 'attribute', 'allowLinking')
    runcontext.set_mod_params_instance(ansible_params, 'operation', 'exclude')
    runcontext.set_mod_params_instance(ansible_params, 'value', 'true')
    module = runcontext.get_module_file('xml_query.py')
    rc1, stdout1, stderr1 = runcontext.invoke_module(module, ansible_params)
    runcontext.log_debug_invoke_result(rc1, stdout1, stderr1)
    runcontext.sleep()

    runcontext.set_mod_params_instance(ansible_params,'path',gcontextxml)
    runcontext.set_mod_params_instance(ansible_params,'xpath','.//{ns}Context')
    runcontext.set_mod_params_instance(ansible_params,'content','attribute')
    runcontext.set_mod_params_instance(ansible_params,'attribute','allowLinking')
    runcontext.set_mod_params_instance(ansible_params,'operation','exclude')
    runcontext.set_mod_params_instance(ansible_params,'value','true')
    module = runcontext.get_module_file('xml_query.py')
    rc2,stdout2,stderr2 = runcontext.invoke_module(module, ansible_params)
    runcontext.log_debug_invoke_result(rc2, stdout2, stderr2)
    runcontext.sleep()

    resultjson1 = json.loads(stdout1)
    resultjson2 = json.loads(stdout2)
    errMsgs = []
    checkMsgs = []

    if 'true' in [x.lower() for x in resultjson1['items']['values']]:
        errMsgs.append('%s : <Context ... allowLinking="true"/>' % gserverxml)
    if 'false' in [x.lower() for x in resultjson1['items']['values']]:
        checkMsgs.append('%s : <Context ... allowLinking="false"/>' % gserverxml)
    if '' in [x.lower() for x in resultjson1['items']['values']]:
        checkMsgs.append('%s : <Context ... />' % gserverxml)

    if 'true' in [x.lower() for x in resultjson2['items']['values']]:
        errMsgs.append('%s : <Context ... allowLinking="true"/>' % gcontextxml)
    if 'false' in [x.lower() for x in resultjson2['items']['values']]:
        checkMsgs.append('%s : <Context ... allowLinking="false"/>' % gcontextxml)
    if '' in [x.lower() for x in resultjson2['items']['values']]:
        checkMsgs.append('%s : <Context ... />' % gcontextxml)

    curtemplate = runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''

    ret_code = -2
    ret_msg = ''

    #get webapps
    webapps = runcontext.get_extra_var('TOMCAT_WEBAPPS')

    if webapps is None:
        return ret_code, 'The webapps is none.'

    for webapp in webapps:
        contexts = webapp['context.xml']
        host_name = webapp['host_name']
        context_path = webapp['context_path']
        
        #context.xml atleast have one
        if len(contexts) < 1:
            errMsg = '%s\n%s' % (errMsg, 'Web App %s%s: context.xml does not exist. ' % (host_name, context_path))
            continue

        #get the configuration value in the contexts in reverse order
        for context in contexts:
            if context == gcontextxml:
                continue
            xmlTree = ET.parse(context)
            nodeContext = xmlTree.getroot()
            if nodeContext.tag != 'Context':
                nodes = xmlTree.findall('.//Context')
                if len(nodes) == 0:
                    continue
                else:
                    #only check the first node
                    nodeContext = nodes[0]

            #if confg
            if 'allowLinking' in nodeContext.attrib:

                value = nodeContext.attrib['allowLinking']

                if value.lower() == 'true':
                    errMsgs.append('Webapp %s%s: <Context ... allowLinking="true"/>' % (host_name, context_path))
                else:
                    checkMsgs.append('Webapp %s%s: <Context ... allowLinking="false"/>' % (host_name, context_path))
            else:
                checkMsgs.append('Webapp %s%s: <Context ... />' % (host_name, context_path))
    
    if len(errMsgs) != 0:
        ret_code = -1
        tmp_vars["checkvalue1"] = '\n'.join(errMsgs)
    else:
        ret_code = 0
        tmp_vars["policyresult"] = "pass"
        tmp_vars["checkresult1"] = "pass"
        tmp_vars["checkvalue1"] = '\n'.join(checkMsgs)
    
    curtemplate = runcontext.get_template()
    rc1, out = curtemplate.render(runcontext.get_template_file('r_2_2_3.j2'), **tmp_vars)

    if rc1 == False:
        ret_code = -2
        ret_msg = ('Failed to render template. %s' % out)
    else:
        ret_msg = out

    return ret_code, ret_msg
