# -*- coding: UTF-8 -*-
import re
try:
    import xml.etree.cElementTree as ET
except ImportError:
    import xml.etree.ElementTree as ET


def get_tags():
    return ['2', '2.1', '2.1.8']


def get_xml_namespace(node):
    pattern = re.compile(r'({\S+})(\S+)')
    m = pattern.match(node)
    if m:
        return m.groups()[0]
    else:
        return ''


def get_node_tag(node):
    pattern = re.compile(r'({\S+})*(\S+)')
    m = pattern.match(node)
    if m:
        return m.groups()[1]
    else:
        return ''


def run_task(runcontext):
    g_server_xml = runcontext.get_extra_var('TOMCAT_GLOBAL_SERVER_FILE')

    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["systemvalue1"] = ''
    ret_code = -1
    ret_msg = ''
    ret_err_msg = []
    ret_ok_msg = []

    xmlTree = ET.parse(g_server_xml)
    root = xmlTree.getroot()
    namespace = get_xml_namespace(root.tag)
    s = './/{0}Realm'.format(namespace)
    #SSLEnabled
    realm_nodes = root.findall(s)
    ret_ok_msg.append('<Server>\n...')
    ret_err_msg.append('<Server>\n...')
    if realm_nodes:
        all_result = []
        for node in realm_nodes:
            node_msg = []
            node_tag = get_node_tag(node.tag)
            attribs_result = [False, False]
            attrib_list = []
            for attrib in node.attrib.keys():
                attrib_value = node.attrib.get(attrib)
                attrib_value = '' if attrib_value is None else attrib_value
                attrib_list.append('{0}="{1}"'.format(attrib, attrib_value))
                if attrib == 'className':
                    if attrib_value == 'org.apache.catalina.realm.LockOutRealm':
                        attribs_result[0] = True
                    else:
                        if attribs_result[0]:
                            pass
                if attrib == 'failureCount':
                    if attrib_value.isdigit() and int(attrib_value) in [1, 2, 3, 4, 5]:
                        attribs_result[1] = True
                    else:
                        if attribs_result[1]:
                            pass

            node_msg.append('    <{0} ... {1} ... >\n    </{0}>'.format(node_tag, ' '.join(attrib_list)))
            if False not in attribs_result:
                all_result.append(True)
                ret_ok_msg.extend(node_msg)
            else:
                all_result.append(False)
                ret_err_msg.extend(node_msg)

        if True in all_result:
            ret_code = 0
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["policyresult"] = "pass"
            ret_ok_msg.append('...\n</Server>')
            tmp_vars["systemvalue1"] = '\n'.join(ret_ok_msg)
        else:
            ret_err_msg.append('...\n</Server>')
            tmp_vars["systemvalue1"] = '\n'.join(ret_err_msg)
    else:
        ret_code = 0
        tmp_vars["systemvalue1"] = 'Realm is not configured'
        tmp_vars["checkresult1"] = "pass"
        tmp_vars["policyresult"] = "pass"

    curtemplate = runcontext.get_template()
    rc1, out = curtemplate.render(runcontext.get_template_file('s_2_1_3.j2'), **tmp_vars)

    if rc1 == False:
        ret_code = -2
        ret_msg = ('Failed to render template. %s' % out)
    else:
        ret_msg = out

    return ret_code, ret_msg
